# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils import timezone
from django.utils import translation
from django.conf import settings
from django.apps import apps

from evotevotings.services.routers import CSrouter
from celery.utils.log import get_task_logger
from celery.schedules import crontab
from evote.celery import app
from django_fsm import can_proceed

from .mailings import ElectionNotificationDateMail

logger = get_task_logger(__name__)


def get_election(pk):
    Election = apps.get_model("evoteelections.Election")  # noqa
    return Election.objects.get(pk=pk)


@app.task(ignore_result=True)
def publish_provisional_roll(election_id):
    """Task to change the election state from open to provisional electoral
    roll"""
    election = get_election(election_id)
    if can_proceed(election.publish_provisional_roll):
        election.publish_provisional_roll()
        election.save()
    return election


@app.task(ignore_result=True)
def publish_definitive_roll(election_id):
    """Task to change the election state from provisional electoral roll to
    definitive electoral roll"""
    election = get_election(election_id)
    if can_proceed(election.publish_definitive_roll):
        election.publish_definitive_roll()
        election.save()
    return election


@app.task(ignore_result=True)
def open_election_voting(election_id):
    """Task to change the election state from definitive electoral roll to
    in to vote"""
    election = get_election(election_id)
    if can_proceed(election.voting):
        election.voting()
        election.save()
    return election


@app.task(ignore_result=True)
def close_election_voting(election_id):
    """Task to change the election state from in to vote to in counting"""
    election = get_election(election_id)
    if can_proceed(election.counting):
        election.counting()
        election.save()
    return election


@app.task(ignore_result=True, bind=True, max_retries=None)
def close_election_counting(self, election_id):
    """Task to change the election state from in counting to votes scrutinized.
    http://docs.celeryproject.org/en/latest/userguide/tasks.html#retrying
    """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    stop_status = [conf.get('counting_finish_state'),
                   conf.get('counting_error_state')]
    status = CSrouter().check_results_status(election_id)
    # counting in progress
    if status is not None and status not in stop_status:
        raise self.retry(countdown=10, max_retries=None)
    # counting finished
    elif status == conf.get('counting_finish_state'):
        election = get_election(election_id)
        if can_proceed(election.scrutinized):
            election.scrutinized()
            election.save()
        return election
    # error or counting not initialize yet
    else:
        # TODO: log this
        pass
    return None


@app.task(ignore_result=True)
def begin_minute_approval(election_id):
    """Task to change the election state from votes scrutinized to
    minute in approval"""
    election = get_election(election_id)
    if can_proceed(election.begin_approval):
        election.begin_approval()
        election.save()
    return election


@app.task(ignore_result=True)
def close_election(election_id):
    """Task to close an election"""
    election = get_election(election_id)
    if can_proceed(election.close):
        election.close()
        election.save()
        electoral_process = election.electoral_process
        if can_proceed(electoral_process.close):
            electoral_process.close()
            electoral_process.save()
    return election


@app.task(ignore_result=True)
def archive_election(election_id):
    """Task to archive an election"""
    election = get_election(election_id)
    if can_proceed(election.archive):
        election.archive()
        election.save()
        electoral_process = election.electoral_process
        if can_proceed(electoral_process.archive):
            electoral_process.archive()
            electoral_process.save()
    return election


@app.on_after_finalize.connect
def setup_periodic_tasks(sender, **kwargs):
    """ """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    # declare send_elections_notifications task
    sender.add_periodic_task(
        crontab(
            hour=conf.get('notification_time')[0],
            minute=conf.get('notification_time')[1]
        ),
        send_elections_notifications.s(),
    )


@app.task
def send_elections_notifications():
    """ """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    sent = 0
    Election = apps.get_model("evoteelections.Election") # NOQA
    qs = Election.objects.filter(state__gte=Election.State.OPEN)
    fields = {
        'proposition_begin_date': {
            'subject': 'Notificação eVotUM - Período de apresentação de' +
                       ' listas candidatas | eVotUM notification - Period' +
                       ' for the submission of applications',
            'type_pt': 'período de apresentação de listas candidatas',
            'type': 'period for the submission of applications',
            'kind_pt': 'inicia-se',
            'kind': 'begins'
        },
        'proposition_end_date': {
            'subject': 'Notificação eVotUM - Período de apresentação de' +
                       ' listas candidatas | eVotUM notification - Period' +
                       ' for the submission of applications',
            'type_pt': 'período de apresentação de listas candidatas',
            'type': 'period for the submission of applications',
            'kind_pt': 'termina',
            'kind': 'ends'
        },
        'minute_end_date': {
            'subject': 'Notificação eVotUM - Período de homologação | ' +
                       'eVotUM notification - Approval period',
            'type_pt': 'período de homologação',
            'type': 'approval period',
            'kind_pt': 'termina',
            'kind': 'ends'
        },
        'voting_begin_date': {
            'subject': 'Notificação eVotUM - Período de votação | ' +
                       'eVotUM notification - Voting time',
            'type_pt': 'período de votação',
            'type': 'voting time',
            'kind_pt': 'inicia-se',
            'kind': 'begins'
        },
        'roll_begin_date': {
            'subject': 'Notificação eVotUM - Período de publicação dos ' +
                       'cadernos eleitorais provisórios | eVotUM ' +
                       'notification - Publication of the provisional ' +
                       'electoral roll',
            'type_pt': 'período de publicação dos cadernos eleitorais'
                       ' provisórios',
            'type': 'publication of the provisional electoral roll',
            'kind_pt': 'termina',
            'kind': 'ends'
        },
        'roll_end_date': {
            'subject': 'Notificação eVotUM - Período de publicação dos ' +
                       'cadernos eleitorais definitivos | eVotUM ' +
                       'notification - Publication of the definitive ' +
                       'electoral roll',
            'type_pt': 'período de publicação dos cadernos eleitorais'
                       ' definitivos',
            'type': 'publication of the definitive electoral roll',
            'kind_pt': 'termina',
            'kind': 'ends'
        }
    }
    days = conf.get('notification_days_before')
    for election in qs:
        for key in fields:
            date = getattr(election, key)
            for day in days:
                notify_date = date - timezone.timedelta(days=day)
                if notify_date.date() == timezone.now().date():
                    ElectionNotificationDateMail(
                        subject=fields[key]['subject'],
                        context={
                            'election_id': election.identifier,
                            'process_id':
                                election.electoral_process.identifier,
                            'days': day,
                            'type_pt': fields[key]['type_pt'],
                            'type': fields[key]['type'],
                            'kind_pt': fields[key]['kind_pt'],
                            'kind': fields[key]['kind']
                        })\
                        .send(election.electoral_process)
                    sent += 1
    return sent


@app.task(ignore_result=True)
def generate_election_pdf(election_id):
    """Task to generate pdf in all languages """
    languages = getattr(settings, 'LANGUAGES', None)
    for lang in languages:
        translation.activate(lang[0])
        election = get_election(election_id)
        election.generate_voters_pdf()
    return election
